/*
* CLIF is a Load Injection Framework
* Copyright (C) 2005 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF $Name: not supported by cvs2svn $
*
* Contact: clif@objectweb.org
*/

package org.objectweb.clif.isac.plugins;

import org.objectweb.clif.scenario.isac.plugin.ControlAction;
import org.objectweb.clif.scenario.isac.plugin.DataProvider;
import org.objectweb.clif.scenario.isac.plugin.TestAction;
import org.objectweb.clif.scenario.isac.plugin.SessionObjectAction;
import org.objectweb.clif.scenario.isac.exception.IsacRuntimeException;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;


/**
 * ISAC plug-in providing a context holder to define and use arbitrary variables/constants
 * @author Bruno Dillenseger
 */
public class ContextSessionObject implements DataProvider, ControlAction, TestAction, SessionObjectAction
{
	static private final char SET_CHAR = '=';
	static private final char SEP_CHAR = ';';
	static private final String VARS_ARG = "variables";
	static private final String VAR_ARG = "variable";
	static private final String VALUE_ARG = "value";
	static private final int ISDEF_TEST = 0;
	static private final int ISNOTDEF_TEST = 1;
	static private final int SET_CONTROL = 0;

	static private void parseVariables(String varstr, Map result)
	{
		int begin = 0;

		while (begin < varstr.length())
		{
			int end = varstr.indexOf(SEP_CHAR, begin);
			if (end == -1)
			{
				end = varstr.length() - 1;
			}
			int middle = varstr.indexOf(SET_CHAR, begin);
			if (middle == -1 || middle >= end)
			{
				result.put(varstr.substring(begin, end), "");
			}
			else
			{
				result.put(varstr.substring(begin, middle), varstr.substring(middle + 1, end));
			}
			begin = end + 1;
		}
	}

	private HashMap variables;
	private HashMap defaultVariables;

	/**
	 * Build a new SessionObject for this plugin
	 * @param params The table containing all the parameters
	 */
	public ContextSessionObject(Hashtable params)
	{
		variables = new HashMap();
		parseVariables((String)params.get(VARS_ARG), variables);
	}


	/**
	 * This constructor is used to clone the specified session object
	 *
	 * @param toClone
	 *            The session object to clone
	 */
	private ContextSessionObject(ContextSessionObject toClone)
	{
		defaultVariables = toClone.variables;
		variables = (HashMap)toClone.variables.clone();
	}


	///////////////////////////
	// TestAction interface //
	///////////////////////////


	public boolean doTest(int number, Map params)
		throws IsacRuntimeException
	{
		switch (number)
		{
			case ISDEF_TEST:
				return variables.containsKey(params.get(VAR_ARG));
			case ISNOTDEF_TEST:
				return ! variables.containsKey(params.get(VAR_ARG));
			default:
				throw new Error("Fatal error in ISAC's Context plug-in: unknown test identifier " + number);
		}
	}

	
	////////////////////////////
	// DataProvider interface //
	////////////////////////////

	
	public String doGet(String var)
	{
		return (String)variables.get(var);
	}


	////////////////////////////
	// ControlAction interface //
	////////////////////////////


	public void doControl(int number, Map params)
	{
		switch (number)
		{
			case SET_CONTROL:
				variables.put(params.get(VAR_ARG), params.get(VALUE_ARG));
				break;
			default:
				throw new Error("Fatal error in ISAC's Context plug-in: unknown control identifier " + number);
		}
	}

	///////////////////////////////////
	// SessionObjectAction interface //
	///////////////////////////////////


	public Object createNewSessionObject()
	{
		return new ContextSessionObject(this);
	}


	public void close()
	{
	}


	public void reset()
	{
		variables = (HashMap)defaultVariables.clone();
	}
}
