/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * CLIF
 *
 * Contact: clif@objectweb.org
 */
package org.objectweb.clif.isac.plugins;

import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.UnknownHostException;
import java.util.Hashtable;
import java.util.Map;
import java.util.Random;

import org.objectweb.clif.scenario.isac.plugin.SampleAction;
import org.objectweb.clif.scenario.isac.plugin.SessionObjectAction;
import org.objectweb.clif.storage.api.ActionEvent;
import org.objectweb.clif.util.Network;
import org.xbill.DNS.Lookup;
import org.xbill.DNS.Record;
import org.xbill.DNS.Resolver;
import org.xbill.DNS.SimpleResolver;


/**
 * DNS injecton plug-in for CLIF/ISAC, based on modified dnsjava library
 *
 * @author Bruno Dillenseger
 */
public class DnsInjector implements SampleAction, SessionObjectAction
{
	static private Random rand = new Random();

	// plug-in parameters
	static public final String SERVER_ARG = "server_arg";
	static public final String PREFIX_ARG = "prefix_arg";
	static public final String SUFFIX_ARG = "suffix_arg";
	static public final String FROM_ARG = "from_arg";
	static public final String TO_ARG = "to_arg";

	// sample identifier
	static public final int QUERY_SAMPLE = 0;

	// session object parameters
	private long from, range;
	private String prefix = "", suffix = "";
	private byte numberOfBytes = 0;
	private String server;
	private Resolver[] resolvers;


	/**
	 * Build a new SessionObject for this plugin
	 * @param params The table containing all the parameters
	 */
	public DnsInjector(Hashtable params)
	{
		String value = (String)params.get(SERVER_ARG);
		if (value != null && value.length() > 0)
		{
			server = value;
		}
		InetAddress[] addresses = Network.getInetAddresses(null);
		resolvers = new Resolver[addresses.length];
		for (int i=0 ; i<addresses.length ; ++i)
		{
			try
			{
				resolvers[i] = new SimpleResolver(server, new InetSocketAddress(addresses[i], 0));
			}
			catch (UnknownHostException ex)
			{
				throw new RuntimeException("Could not instantiate DnsInjector session object: " + ex);
			}
		}
		value = (String)params.get(PREFIX_ARG);
		if (value != null)
		{
			prefix = value;
		}
		value = (String)params.get(SUFFIX_ARG);
		if (value != null)
		{
			suffix = value;
		}
		value = (String)params.get(FROM_ARG);
		if (value != null)
		{
			from = bytes2long(Long.parseLong(value));
		}
		long to = 0;
		value = (String)params.get(TO_ARG);
		if (value != null)
		{
			to = bytes2long(Long.parseLong(value));
			numberOfBytes = (byte) ((value.length() + 2) / 3);
		}
		range = 1 + to - from;
	}


	private long bytes2long(long value)
	{
		if (value == 0)
		{
			return 0;
		}
		else
		{
			return (255 * bytes2long(value/1000)) + (value % 1000);
		}
	}


	/**
	 * This constructor is used to clone the specified session object
	 *
	 * @param toClone
	 *            The session object to clone
	 */
	private DnsInjector(DnsInjector toClone)
	{
		prefix = toClone.prefix;
		suffix = toClone.suffix;
		range = toClone.range;
		from = toClone.from;
		server = toClone.server;
		resolvers = toClone.resolvers;
		numberOfBytes = toClone.numberOfBytes;
	}


	////////////////////////////
	// SampleAction interface //
	////////////////////////////


	public ActionEvent doSample(int number, Map params, ActionEvent event)
	{
		switch (number)
		{
			case QUERY_SAMPLE:
				long value = from + (Math.abs(rand.nextLong()) % range);
				String name = suffix;
				for (int i=0 ; i<numberOfBytes ; ++i)
				{
					short octet = (short)(value % 255);
					name = String.valueOf(octet) + name;
					if (octet < 10)
					{
						name = "00" + name;
					}
					else if (octet < 100)
					{
						name = "0" + name;
					}
					value /= 255;
				}
				name = prefix + name;
				event.type = "type A DNS query";
				try
				{
					int index = rand.nextInt(resolvers.length);
					event.comment = name + " from " + Network.getInetAddresses(null)[index];
					Lookup lookup = new Lookup(name);
					lookup.setCache(null);
					lookup.setResolver(resolvers[index]);
					event.setDate(System.currentTimeMillis());
					Record[] results = lookup.run();
					event.duration = (int)(System.currentTimeMillis() - event.getDate());
					if (results != null && results.length > 0)
					{
						event.successful = true;
						event.result = results[0].rdataToString();
					}
					else
					{
						event.successful = false;
						event.result = lookup.getResult() + " - " + lookup.getErrorString();
					}
				}
				catch (Throwable ex)
				{
					event.successful = false;
					event.result = ex.toString();
					event.duration = 0;
				}
				return event;
			default:
				throw new Error("Unknown sample number " + number);
		}
	}


	///////////////////////////////////
	// SessionObjectAction interface //
	///////////////////////////////////


	public Object createNewSessionObject()
	{
		return new DnsInjector(this);
	}


	public void close()
	{
	}


	public void reset()
	{
	}
}
