/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2004, 2005, 2006 France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * CLIF
 *
 * Contact: clif@objectweb.org
 */

package org.objectweb.clif.isac.plugins.httpinjector10.actions;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashSet;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.Map;
import org.htmlparser.Parser;
import org.htmlparser.Node;
import org.htmlparser.tags.ImageTag;
import org.htmlparser.tags.FrameTag;
import org.htmlparser.util.ParserException;
import org.apache.commons.httpclient.Header;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.httpclient.URIException;
import org.apache.commons.httpclient.UsernamePasswordCredentials;
import org.apache.commons.httpclient.URI;
import org.apache.commons.httpclient.auth.AuthScope;
import org.apache.commons.httpclient.methods.DeleteMethod;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.HeadMethod;
import org.apache.commons.httpclient.methods.InputStreamRequestEntity;
import org.apache.commons.httpclient.methods.OptionsMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.PutMethod;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.apache.commons.httpclient.methods.multipart.FilePart;
import org.apache.commons.httpclient.methods.multipart.MultipartRequestEntity;
import org.apache.commons.httpclient.methods.multipart.Part;
import org.objectweb.clif.isac.plugins.httpinjector10.SessionObject;
import org.objectweb.clif.isac.plugins.httpinjector10.tools.HostConfigurationUtils;
import org.objectweb.clif.isac.plugins.httpinjector10.tools.HttpStateUtils;
import org.objectweb.clif.isac.plugins.httpinjector10.tools.ParameterConstants;
import org.objectweb.clif.scenario.isac.exception.IsacRuntimeException;
import org.objectweb.clif.storage.api.ActionEvent;
import org.objectweb.clif.util.ClifClassLoader;

/**
 * This class implements the HttpInjector sample methods.
 *
 * @author JC Meillaud
 * @author A Peyrard
 * @author Emmanuel Varoquaux
 * @author Bruno Dillenseger
 */
public class HttpInjectorSamples {
	public static final int	MAXATTEMPT	= 1;

	/**
	 * Executes a previously initialized HttpMethod.
	 *
	 * @param sessionObject
	 *            The sessionObject which handles the httpClient.
	 * @param report
	 *            The ActionReport to update..
	 * @param httpMethod
	 *            the HttpMethod to execute
	 * @param responseVar if not null, the response body is stored as a variable 
	 * with the given name in the session object(@see DataProvider)
	 */
	private static void executeMethod(SessionObject sessionObject,
			ActionEvent report, HttpMethod httpMethod, String responseVar) {
		final HttpClient httpClient = sessionObject.getHttpClient();
		int responseCode;
		Header[] responseHeaders;
		String responseBody;
		String msg;

		responseCode = -1;
		responseHeaders = null;
		responseBody = null;
		msg = null;
		httpMethod.addRequestHeader("User-Agent", sessionObject.getUserAgent());
		report.setDate(System.currentTimeMillis());
		for (int attempt = 0; responseCode < 0 && attempt < MAXATTEMPT; attempt++) {
			try {
				responseCode = httpClient.executeMethod(httpMethod);
				msg = httpMethod.getStatusLine().toString();
				responseHeaders = httpMethod.getResponseHeaders();
				if (sessionObject.inDepthLoadEnabled() || responseVar != null)
				{
					responseBody = httpMethod.getResponseBodyAsString();
				}
				if (responseVar != null)
				{
					sessionObject.setVariable(responseVar, responseBody);
				}
				report.duration = (int)(System.currentTimeMillis() - report.getDate());
			}
			catch (IOException e) {
				msg = e.getMessage();
			}
			finally {
				httpMethod.releaseConnection();
			}
		}
		sessionObject.setLastStatusCode(responseCode);
		sessionObject.setLastHeader(responseHeaders);
		sessionObject.setLastResponseBody(responseBody);
		report.result = msg;
		if (responseBody != null && sessionObject.inDepthLoadEnabled())
		{
			try
			{
				inDepthLoad(sessionObject, responseBody, httpMethod.getURI());
			}
			catch (URIException e) {}
		}
		report.successful = responseCode >= 0;
	}

	/**
	 * Sets the credentials for a particular host and realm.
	 *
	 * @param sessionObject
	 *            The sessionObject.
	 * @param params
	 *            Map containing all variables.
	 */
	private static void setCredentials(SessionObject sessionObject, Map params) {
		String hostAuth, realm;
		String username, password;

		if ((hostAuth = (String)params.get(ParameterConstants.HOSTAUTH)) != null
				&& hostAuth.equals("null"))
			hostAuth = null;
		if ((realm = (String)params.get(ParameterConstants.REALM)) != null
				&& realm.equals("null"))
			realm = null;
		username = (String)params.get(ParameterConstants.USERNAME);
		password = (String)params.get(ParameterConstants.PASSWORD);
		if (username != null && !username.equals(""))
		{
			sessionObject.getHttpClient().getState().setCredentials(
					new AuthScope(hostAuth, AuthScope.ANY_PORT, realm),
					new UsernamePasswordCredentials(username, password));
		}
	}

	/**
	 * Builds an array of name/value pairs and returns this array,
	 * that will be set into the query string.
	 *
	 * @param inputParameters
	 *            a String containing all the parameters in a serialized way
	 * @return An array of name/value pairs.
	 */
	private static NameValuePair[] setInputParameters(String inputParameters)
	{
		Set<NameValuePair> res = new HashSet<NameValuePair>();
		
		StringTokenizer st, st2;

		st = new StringTokenizer(inputParameters, ";");
		while (st.hasMoreTokens())
		{
			st2 = new StringTokenizer(st.nextToken(), "=");
			if (st2.countTokens() == 2) {
				String inputParamName = st2.nextToken();
				String inputParamValue = st2.nextToken();
				if (!inputParamName.equals("") && !inputParamValue.equals(""))
				{
					res.add(new NameValuePair(inputParamName, inputParamValue));
				}
			}
			else if (st2.countTokens() == 1)
			{
				String inputParamName = st2.nextToken();
				if (!inputParamName.equals(""))
				{
					res.add(new NameValuePair(inputParamName, ""));
				}
			}
		}
		return res.toArray(new NameValuePair[res.size()]);
	}


	private static String completeUri(String uri, Map<String,String> params) {
		String scheme = uri.substring(0, 5);
		if (!scheme.equals("http:") && !scheme.equals("https"))
			params.put(ParameterConstants.URI, uri = "http://" + uri);
		return uri;
	}

	/**
	 * Loads frames (recursively) and images included in a given HTML body.
	 * 
	 * @param session the session object.
	 * @param body the HTML text.
	 * @param baseURI the URI of this HTML text (necessary to resolve relative pathes).
	 */
	static private long inDepthLoad(SessionObject session, String body, URI baseURI) {
		long startTime;
		boolean isFrameSet;
		Parser parser;
		Node[] nodes;
		GetMethod method;
		String rootPath;
		String responseBody;

		parser = Parser.createParser(body);
		try {
			nodes = parser.extractAllNodesThatAre(FrameTag.class);
		}
		catch (ParserException e) {
			return 0;
		}
		isFrameSet = nodes.length > 0;
		if (!isFrameSet) {
			parser.reset();
			try {
				nodes = parser.extractAllNodesThatAre(ImageTag.class);
			}
			catch (ParserException e) {
				return 0;
			}
		}
		try {
			rootPath = baseURI.getPath().substring(0, 1 + baseURI.getPath().lastIndexOf('/'));
		}
		catch (URIException e) {
			return 0;
		}
		startTime = System.currentTimeMillis();
		for (int i = 0; i < nodes.length; i++)
		{
			String path;
			path = isFrameSet
				? ((FrameTag)nodes[i]).getFrameLocation()
				: ((ImageTag)nodes[i]).extractImageLocn();
			if (path.startsWith("http:") || path.startsWith("https:")
				|| path.startsWith("ftp:") || path.startsWith("file:"))
			{
				method = new GetMethod(path);
			}
			else
			{
				method = new GetMethod(baseURI.toString());
				if (path.startsWith("/"))
					method.setPath(path);
				else
					method.setPath(rootPath + path);
			}
			try {
				session.getHttpClient().executeMethod(method);
			}
			catch (IOException e2) {
				continue;
			}
			finally {
				method.releaseConnection();
			}
			try
			{
				if (isFrameSet
					&& (responseBody = method.getResponseBodyAsString()) != null)
				{
					inDepthLoad(session, responseBody, method.getURI());
				}
			}
			catch (Exception ex)
			{
				throw new RuntimeException("Exception while loading frame " + path + " in " + baseURI, ex);
			}
		}
		return System.currentTimeMillis() - startTime;
	}

	/**
	 * Does a HTTP GET.
	 *
	 * @param sessionObject
	 *            The sessionObject which handles the httpClient.
	 * @param report
	 *            The ActionReport to update.
	 * @param params
	 *            A Map containing all the useful variable.
	 * @return the report.
	 */
	public static ActionEvent doGet(SessionObject sessionObject,
			ActionEvent report, Map<String,String> params) {
		final HttpClient httpClient = sessionObject.getHttpClient();
		String uri, parameters, redirect;
		GetMethod method;

		report.type = "HTTP GET";
		if (!params.containsKey(ParameterConstants.URI)) {
			report.result = "Undefined URI";
			report.successful = false;
			return report;
		}
		uri = completeUri((String)params.get(ParameterConstants.URI), params);
		report.comment = uri;
		parameters = (String)params.get(ParameterConstants.QUERYPARAMETERS);
		redirect = (String)params.get(ParameterConstants.REDIRECT);
		if (params.containsKey(ParameterConstants.USERNAME) && params.containsKey(ParameterConstants.PASSWORD))
		{
			setCredentials(sessionObject, params);
		}
		method = new GetMethod(uri);
		if (parameters != null && !parameters.equals(""))
		{
			method.setQueryString(setInputParameters(parameters));
		}
		method.getParams().makeLenient();
		if (redirect != null)
		{
			method.setFollowRedirects(redirect.equals(ParameterConstants.ENABLED));
		}
		httpClient.setState(HttpStateUtils.setHttpState(sessionObject, params));
		try {
			httpClient.setHostConfiguration(HostConfigurationUtils
					.setHostConfiguration(sessionObject, params));
		}
		catch (URIException e) {
			report.result = "Malformed URI";
			report.successful = false;
			return report;
		}
		setHeaders(method, (String)params.get(ParameterConstants.HEADERS));
		String responseBodyVariableName = (String)params.get(ParameterConstants.RESPONSE_BODY);
		if (responseBodyVariableName != null && responseBodyVariableName.length() == 0)
		{
			responseBodyVariableName = null;
		}
		executeMethod(sessionObject, report, method, responseBodyVariableName);
		return report;
	}

	/**
	 * Does a HTTP POST.
	 *
	 * @param sessionObject
	 *            The sessionObject which handles the httpClient.
	 * @param report
	 *            The ActionReport to update.
	 * @param params
	 *            A Map containing all the useful variable.
	 * @return the report.
	 */
	public static ActionEvent doPost(SessionObject sessionObject,
			ActionEvent report, Map<String,String> params) {
		final HttpClient httpClient = sessionObject.getHttpClient();
		String uri, queryParameters, bodyParameters, redirect, request, file;
		PostMethod method;

		report.type = "HTTP POST";
		if (!params.containsKey(ParameterConstants.URI)) {
			report.comment = "Undefined URI";
			report.successful = false;
			return report;
		}
		uri = completeUri((String)params.get(ParameterConstants.URI), params);
		report.comment = uri;
		queryParameters = (String)params.get(ParameterConstants.QUERYPARAMETERS);
		bodyParameters = (String)params.get(ParameterConstants.BODYPARAMETERS);
		redirect = (String)params.get(ParameterConstants.REDIRECT);
		request = (String)params.get(ParameterConstants.REQUEST);
		file = (String)params.get(ParameterConstants.FILE);
		if (params.containsKey(ParameterConstants.USERNAME) && params.containsKey(ParameterConstants.PASSWORD))
			setCredentials(sessionObject, params);
		method = new PostMethod(uri);
		if (queryParameters != null && !queryParameters.equals(""))
		{
			method.setQueryString(setInputParameters(queryParameters));
		}
		if (bodyParameters != null && !bodyParameters.equals(""))
		{
			method.setRequestBody(setInputParameters(bodyParameters));
		}
		if (file != null && !file.equals("")) {
			InputStream fileInputStream;
			if ((fileInputStream = ClifClassLoader.getClassLoader()
					.getResourceAsStream(file)) != null)
			{
				method.setRequestEntity(new InputStreamRequestEntity(fileInputStream));
			}
		}
		if (request != null && !request.equals(""))
		{
			try
			{
				method.setRequestEntity(new StringRequestEntity(request));
			}
			catch (Exception ex)
			{
				throw new IsacRuntimeException("Exception when getting parameters for HTTP POST", ex);
			}
		}
		method.getParams().makeLenient();
		if (redirect != null)
			method.setFollowRedirects(redirect.equals(ParameterConstants.ENABLED));

		httpClient.setState(HttpStateUtils.setHttpState(sessionObject, params));
		try {
			httpClient.setHostConfiguration(HostConfigurationUtils
					.setHostConfiguration(sessionObject, params));
		}
		catch (URIException e)
		{
			report.result = "Malformed URI";
			report.successful = false;
			return report;
		}
		setHeaders(method, (String)params.get(ParameterConstants.HEADERS));
		String responseBodyVariableName = (String)params.get(ParameterConstants.RESPONSE_BODY);
		if (responseBodyVariableName != null && responseBodyVariableName.length() == 0)
		{
			responseBodyVariableName = null;
		}
		executeMethod(sessionObject, report, method, responseBodyVariableName);
		return report;
	}

	/**
	 * Does a HTTP MULTIPOST.
	 *
	 * @param sessionObject
	 *            The sessionObject which handles the httpClient.
	 * @param report
	 *            The ActionReport to update.
	 * @param params
	 *            A Map containing all the useful variable.
	 * @return the report.
	 */
	public static ActionEvent doMultiPost(SessionObject sessionObject,
			ActionEvent report, Map<String,String> params) {
		final HttpClient httpClient = sessionObject.getHttpClient();
		String uri, parameters, redirect, files;
		PostMethod method;

		report.type = "HTTP MULTIPOST";
		if (!params.containsKey(ParameterConstants.URI)) {
			report.comment = "Undefined URI";
			report.successful = false;
			return report;
		}
		uri = completeUri((String)params.get(ParameterConstants.URI), params);
		report.comment = uri;
		parameters = (String)params.get(ParameterConstants.QUERYPARAMETERS);
		redirect = (String)params.get(ParameterConstants.REDIRECT);
		files = (String)params.get(ParameterConstants.FILE);
		if (params.containsKey(ParameterConstants.USERNAME) && params.containsKey(ParameterConstants.PASSWORD))
			setCredentials(sessionObject, params);
		method = new PostMethod(uri);
		if (parameters != null && !parameters.equals(""))
			method.setQueryString(setInputParameters(parameters));
		if (files != null && !files.equals(""))
		{
			StringTokenizer st = new StringTokenizer(files, ";");
			Part[] parts;
			parts = new Part[st.countTokens()];
			int i=0;
			while (st.hasMoreTokens())
			{
				String file = st.nextToken();
				try
				{
					File f = new File(file);
					parts[i] = new FilePart(f.getName(), f);
				}
				catch (Exception ex)
				{
					throw new RuntimeException(uri + " Multipart Post failed:" + ex);
				}
				method.setRequestEntity(new MultipartRequestEntity(parts, method.getParams()));
			}
		}
		method.getParams().makeLenient();
		if (redirect != null)
			method.setFollowRedirects(redirect.equals(ParameterConstants.ENABLED));

		httpClient.setState(HttpStateUtils.setHttpState(sessionObject, params));
		try {
			httpClient.setHostConfiguration(HostConfigurationUtils
					.setHostConfiguration(sessionObject, params));
		}
		catch (URIException e)
		{
			report.result = "Malformed URI";
			report.successful = false;
			return report;
		}
		executeMethod(sessionObject, report, method, null);
		return report;
	}

	/**
	 * Does a HTTP HEAD.
	 *
	 * @param sessionObject
	 *            The sessionObject which handles the httpClient.
	 * @param report
	 *            The ActionReport to update.
	 * @param params
	 *            A Map containing all the useful variable.
	 * @return the report.
	 */
	public static ActionEvent doHead(SessionObject sessionObject,
			ActionEvent report, Map<String,String> params) {
		final HttpClient httpClient = sessionObject.getHttpClient();
		String uri, parameters, redirect;
		HeadMethod method;

		report.type = "HTTP HEAD";
		if (!params.containsKey(ParameterConstants.URI)) {
			report.comment = "Undefined URI";
			report.successful = false;
			return report;
		}
		uri = completeUri((String)params.get(ParameterConstants.URI), params);
		report.comment = uri;
		parameters = (String)params.get(ParameterConstants.QUERYPARAMETERS);
		redirect = (String)params.get(ParameterConstants.REDIRECT);
		if (params.containsKey(ParameterConstants.USERNAME) && params.containsKey(ParameterConstants.PASSWORD))
			setCredentials(sessionObject, params);
		method = new HeadMethod(uri);
		if (parameters != null && !parameters.equals(""))
			method.setQueryString(setInputParameters(parameters));
		method.getParams().makeLenient();
		if (redirect != null)
			method.setFollowRedirects(redirect.equals(ParameterConstants.ENABLED));

		httpClient.setState(HttpStateUtils.setHttpState(sessionObject, params));
		try {
			httpClient.setHostConfiguration(HostConfigurationUtils
					.setHostConfiguration(sessionObject, params));
		}
		catch (URIException e) {
			report.result = "Malformed URI";
			report.successful = false;
			return report;
		}
		executeMethod(sessionObject, report, method, null);
		return report;
	}

	/**
	 * Does a HTTP OPTIONS.
	 *
	 * @param sessionObject
	 *            The sessionObject which handles the httpClient.
	 * @param report
	 *            The ActionReport to update.
	 * @param params
	 *            A Map containing all the useful variable.
	 * @return the report.
	 */
	public static ActionEvent doOptions(SessionObject sessionObject,
			ActionEvent report, Map<String,String> params) {
		final HttpClient httpClient = sessionObject.getHttpClient();
		String uri, parameters, redirect;
		OptionsMethod method;

		report.type = "HTTP OPTIONS";
		if (!params.containsKey(ParameterConstants.URI)) {
			report.comment = "Undefined URI";
			report.successful = false;
			return report;
		}
		uri = completeUri((String)params.get(ParameterConstants.URI), params);
		report.comment = uri;
		parameters = (String)params.get(ParameterConstants.QUERYPARAMETERS);
		redirect = (String)params.get(ParameterConstants.REDIRECT);
		if (params.containsKey(ParameterConstants.USERNAME) && params.containsKey(ParameterConstants.PASSWORD))
			setCredentials(sessionObject, params);
		method = new OptionsMethod(uri);
		if (parameters != null && !parameters.equals(""))
			method.setQueryString(setInputParameters(parameters));
		method.getParams().makeLenient();
		if (redirect != null)
			method.setFollowRedirects(redirect.equals(ParameterConstants.ENABLED));

		httpClient.setState(HttpStateUtils.setHttpState(sessionObject, params));
		try {
			httpClient.setHostConfiguration(HostConfigurationUtils
					.setHostConfiguration(sessionObject, params));
		}
		catch (URIException e) {
			report.result = "Malformed URI";
			report.successful = false;
			return report;
		}
		executeMethod(sessionObject, report, method, null);
		return report;
	}

	/**
	 * Does a HTTP PUT.
	 *
	 * @param sessionObject
	 *            The sessionObject which handles the httpClient.
	 * @param report
	 *            The ActionReport to update.
	 * @param params
	 *            A Map containing all the useful variable.
	 * @return the report.
	 */
	public static ActionEvent doPut(SessionObject sessionObject,
			ActionEvent report, Map<String,String> params) {
		final HttpClient httpClient = sessionObject.getHttpClient();
		String uri, parameters, redirect, file;
		PutMethod method;

		report.type = "HTTP PUT";
		if (!params.containsKey(ParameterConstants.URI)) {
			report.comment = "Undefined URI";
			report.successful = false;
			return report;
		}
		uri = completeUri((String)params.get(ParameterConstants.URI), params);
		report.comment = uri;
		parameters = (String)params.get(ParameterConstants.QUERYPARAMETERS);
		redirect = (String)params.get(ParameterConstants.REDIRECT);
		file = (String)params.get(ParameterConstants.FILE);
		if (params.containsKey(ParameterConstants.USERNAME)
			&& params.containsKey(ParameterConstants.PASSWORD))
		{
			setCredentials(sessionObject, params);
		}
		method = new PutMethod(uri);
		if (parameters != null && !parameters.equals(""))
		{
			method.setQueryString(setInputParameters(parameters));
		}
		method.getParams().makeLenient();
		if (redirect != null)
		{
			method.setFollowRedirects(redirect.equals(ParameterConstants.ENABLED));
		}
		if (file != null && !file.equals(""))
		{
			InputStream fileInputStream;
			if ((fileInputStream = ClifClassLoader.getClassLoader().getResourceAsStream(file)) != null)
			{
				method.setRequestEntity(new InputStreamRequestEntity(fileInputStream));
			}
		}

		httpClient.setState(HttpStateUtils.setHttpState(sessionObject, params));
		try {
			httpClient.setHostConfiguration(HostConfigurationUtils
					.setHostConfiguration(sessionObject, params));
		}
		catch (URIException e) {
			report.result = "Malformed URI";
			report.successful = false;
			return report;
		}
		executeMethod(sessionObject, report, method, null);
		return report;
	}

	/**
	 * Does a HTTP DELETE.
	 *
	 * @param sessionObject
	 *            The sessionObject which handles the httpClient.
	 * @param report
	 *            The ActionReport to update.
	 * @param params
	 *            A Map containing all the useful variable.
	 * @return the report.
	 */
	public static ActionEvent doDelete(SessionObject sessionObject,
			ActionEvent report, Map<String,String> params) {
		final HttpClient httpClient = sessionObject.getHttpClient();
		String uri, parameters, redirect;
		DeleteMethod method;

		report.type = "HTTP DELETE";
		if (!params.containsKey(ParameterConstants.URI)) {
			report.comment = "Undefined URI";
			report.successful = false;
			return report;
		}
		uri = completeUri((String)params.get(ParameterConstants.URI), params);
		parameters = (String)params.get(ParameterConstants.QUERYPARAMETERS);
		redirect = (String)params.get(ParameterConstants.REDIRECT);
		if (params.containsKey(ParameterConstants.USERNAME) && params.containsKey(ParameterConstants.PASSWORD))
			setCredentials(sessionObject, params);
		method = new DeleteMethod(uri);
		if (parameters != null && !parameters.equals(""))
			method.setQueryString(setInputParameters(parameters));
		method.getParams().makeLenient();
		if (redirect != null)
			method.setFollowRedirects(redirect.equals(ParameterConstants.ENABLED));

		httpClient.setState(HttpStateUtils.setHttpState(sessionObject, params));
		try {
			httpClient.setHostConfiguration(HostConfigurationUtils
					.setHostConfiguration(sessionObject, params));
		}
		catch (URIException e)
		{
			report.comment = "Malformed URI";
			report.successful = false;
			return report;
		}

		report.comment = uri;
		executeMethod(sessionObject, report, method, null);
		return report;
	}


	/**
	 * Sets headers in the provided HTTP method from the content of a given string.
	 * @param method the target HTTP method object
	 * @param headers a string that must be formatted in a specific way in order to
	 * be parsed as a sequence of header definitions:
	 * "header=header1|value=value1|;header=header2|value=value2...|;header=headerN|value=valueN|;"
	 */
	static void setHeaders(HttpMethod method, String headers)
	{
		if (headers != null)
		{
			StringTokenizer parser = new StringTokenizer(headers, "|");
			String header;
			while (parser.hasMoreTokens() && !(header = parser.nextToken()).equals(";"))
			{
				String value = parser.nextToken();
				method.addRequestHeader(
					header.substring(header.indexOf('=') + 1),
					value.substring(value.indexOf('=') + 1));
			}
		}
	}
}
