/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2004, 2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * CLIF
 *
 * Contact: clif@objectweb.org
 */

package org.objectweb.clif.isac.plugins.httpmatrix10;

import java.util.Hashtable;
import java.util.Map;
import org.apache.commons.httpclient.HostConfiguration;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.MultiThreadedHttpConnectionManager;
import org.objectweb.clif.isac.plugins.httpmatrix10.actions.MatrixSamples;
import org.objectweb.clif.isac.plugins.httpmatrix10.actions.MatrixTests;
import org.objectweb.clif.isac.plugins.httpmatrix10.actions.MatrixTimers;
import org.objectweb.clif.scenario.isac.exception.IsacRuntimeException;
import org.objectweb.clif.scenario.isac.plugin.SampleAction;
import org.objectweb.clif.scenario.isac.plugin.TestAction;
import org.objectweb.clif.scenario.isac.plugin.TimerAction;
import org.objectweb.clif.scenario.isac.plugin.SessionObjectAction;
import org.objectweb.clif.scenario.transitions.TransitionTable;
import org.objectweb.clif.storage.api.ActionEvent;

/**
 * This class is the implementation of a session object for the http matrix
 * plugin
 *
 * @author JC Meillaud
 * @author A Peyrard
 * @author Bruno Dillenseger
 */
public class SessionObject implements SampleAction, TimerAction, TestAction, SessionObjectAction {
	// logger
	//	public static Category cat = Category.getInstance("SessionObject") ;
	// sample constantes definition
	public static final int SAMPLE_DO_NEXT = 0;

	// timer constantes definition
	public static final int TIMER_WAITING_TIME = 0;

	// test constantes definition
	public static final int TEST_HAS_NEXT = 1;

	// test constantes definition
	public static final int TEST_HAS_PREVIOUS = 0;

	// session object params name
	public static final String FILE_MATRIX = "filematrix" ;
	public static final String FILE_ACTIONS = "fileactions";
	public static final String HOST = "host" ;
	public static final String PORT = "port" ;

	// matrix description
	private TransitionTable transitionTable ;

	// http client will be used to execute the http methods
	private HttpClient httpClient;

	// host and port for the proxy
	private String host ;
	private String port ;

	// finish state
	private boolean finishState ;

	/**
	 * Constructor
	 *
	 * @param params
	 *            The table containing all the so params
	 */
	public SessionObject(Hashtable params) {
		// init the http client
		this.httpClient = new HttpClient(new MultiThreadedHttpConnectionManager());
		// init port and host
		this.port = (String)params.get(PORT);
		this.host = (String)params.get(HOST);
		// if the proxy has been configured set it to
		// the connection manager
		try {
			if (!host.equals("") && !port.equals("")) {
				this.setProxy(host, (new Integer(port)).intValue());
			}
		} catch (NullPointerException ne) {
			System.err.println("Unable to set the proxy...") ;
		} catch (NumberFormatException nfe) {
			System.err.println("Unable to set the proxy, the port seams to be malformed");
		}

		// get the matrix file
		String fileName = (String)params.get(FILE_MATRIX);
		// get the actions file
		String fileActions = (String)params.get(FILE_ACTIONS);
		// load the transition table
		this.transitionTable = new TransitionTable(fileName, fileActions, false, false);
		// init the finish state
		this.finishState = false;
	}

	/**
	 * Constructor used in cloneObject method
	 * @param so The session object to clone
	 */
	private SessionObject(SessionObject so) {
		// init the http client
		// The httpClient will be shared between all the session object
		this.httpClient = so.getHttpClient() ;
		// init the matrix description
		this.transitionTable = so.getTransitionTable().createNewTransitionTable() ;
		// init finish state
		this.finishState = false;
	}


	////////////////////////////
	// interface SampleAction //
	////////////////////////////


	public ActionEvent doSample(int number, Map params, ActionEvent report) {
		// switch between the existing sample method
		switch (number) {
		case SAMPLE_DO_NEXT:
			return MatrixSamples.doNext(this, params, report);
		default:
			throw new IsacRuntimeException(
					"Unable to find this sample in matrix plugin : " + number);
		}
	}


	///////////////////////////
	// interface TimerAction //
	///////////////////////////


	public long doTimer(int number, Map params) {
		// switch between the existing timer method
		switch (number) {
		case TIMER_WAITING_TIME:
			return MatrixTimers.waitingTime(this);
		default:
			throw new IsacRuntimeException(
					"Unable to find this timer in matrix plugin : " + number);
		}
	}


	//////////////////////////
	// interface TestAction //
	//////////////////////////


	public boolean doTest(int number, Map params) {
		// switch between the existing test method
		switch (number) {
		case TEST_HAS_NEXT:
			return MatrixTests.hasNext(this);
		case TEST_HAS_PREVIOUS:
			return MatrixTests.hasNext(this);
		default:
			throw new IsacRuntimeException(
					"Unable to find this test in matrix plugin : " + number);
		}
	}

	/**
	 * Method which configure the proxy
	 *
	 * @param host
	 *            The hostname of the proxy server
	 * @param port
	 *            The port name to go through the proxy
	 */
	private void setProxy(String host, int port) {
		// configure the proxy
		HostConfiguration hostConfiguration = new HostConfiguration();
		hostConfiguration.setProxy(host, port);
		// add it to the connection manager
		this.httpClient.setHostConfiguration(hostConfiguration);
	}

	/////////////////////////////////////////
	// SessionObjectAction implementation
	/////////////////////////////////////////

	/**
	 * @see org.objectweb.clif.scenario.isac.plugin.SessionObjectAction#createNewSessionObject()
	 */
	public Object createNewSessionObject() {
		return new SessionObject(this) ;
	}

	/**
	 * @see org.objectweb.clif.scenario.isac.plugin.SessionObjectAction#close()
	 */
	public void close() {
		// nothing to close
	}

	/**
	 * @see org.objectweb.clif.scenario.isac.plugin.SessionObjectAction#reset()
	 */
	public void reset() {
		// reset the states stored in transition table
		this.transitionTable.resetPreviousState();
		// reset finish state
		this.finishState = false;
	}

	///////////////////////////////////////////////////
	// Attribute getters and setters
	///////////////////////////////////////////////////

	/**
	 * @return Returns the httpClient.
	 */
	public HttpClient getHttpClient() {
		return httpClient;
	}

	/**
	 * @param httpClient
	 *            The httpClient to set.
	 */
	public void setHttpClient(HttpClient httpClient) {
		this.httpClient = httpClient;
	}

	/**
	 * @return Returns the transitionTable.
	 */
	public TransitionTable getTransitionTable() {
		return transitionTable;
	}

	/**
	 * @return Returns the host.
	 */
	public String getHost() {
		return host;
	}
	/**
	 * @return Returns the port.
	 */
	public String getPort() {
		return port;
	}

	/**
	 * @return Returns the finishState.
	 */
	public boolean isFinishState() {
		return finishState;
	}
	/**
	 * @param finishState The finishState to set.
	 */
	public void setFinishState(boolean finishState) {
		this.finishState = finishState;
	}
	///////////////////////////////////////
	// Debug Part
	///////////////////////////////////////

//	// this method is only used in debug mode
//	private static void waitTime(int milli) {
//		try {
//			System.out.print("waiting_time=" + milli);
//			for (int i = 0; i < milli; i++) {
//				// each 100 millisec wait 100 millisec
//				if (i % 100 == 0) {
//					System.out.print(".");
//					System.out.flush();
//					Thread.sleep(100);
//				}
//			}
//		} catch (Exception e) {
//			System.out.println("Exception occurs during waiting time...");
//		}
//	}

//	// test method main
//	public static void main(String[] args) {
//		// check if a filename has been set into parameters
//		if (args.length == 0) {
//			System.out.println("usage : java SessionObject matrixFileName");
//		}
//		// proxy configuration
//		String host = "";
//		String port = "";
//		if (args.length == 3) {
//			System.out.println("ask for proxy configuration...");
//			host = args[1];
//			port = args[2];
//		}
//		// init a new Session object
//		SessionObject sessionObject = new SessionObject(args[0], host, port);
//		//		System.out.println(sessionObject.getMatrixDescription()) ;
//		System.out.println("Session object intialized");
//
//		// do n loop
//		int n = 25;
//		for (int i = 0; i < 10; i++) {
//			ActionEvent report = sessionObject.doSample(SAMPLE_DO_NEXT, null);
//			System.out.println(report);
//			long waitingTime = sessionObject.doTimer(TIMER_WAITING_TIME, null);
//			waitTime(waitingTime);
//			boolean test = sessionObject.doTest(TEST_HAS_NEXT, null);
//			System.out.println("continue=" + test);
//			if (!test)
//				break;
//		}
//	}
}
