/*
* CLIF is a Load Injection Framework
* Copyright (C) 2005, 2006 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF $Name$
*
* Contact: clif@objectweb.org
*/

package org.objectweb.clif.isac.plugins;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.EOFException;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import org.objectweb.clif.scenario.isac.exception.IsacRuntimeException;
import org.objectweb.clif.scenario.isac.plugin.ControlAction;
import org.objectweb.clif.scenario.isac.plugin.DataProvider;
import org.objectweb.clif.scenario.isac.plugin.SampleAction;
import org.objectweb.clif.scenario.isac.plugin.TestAction;
import org.objectweb.clif.scenario.isac.plugin.SessionObjectAction;
import org.objectweb.clif.storage.api.ActionEvent;
import org.objectweb.clif.supervisor.api.ClifException;
import org.objectweb.clif.util.Network;

/**
 * ISAC plug-in providing a TCP socket injector
 * @author Bruno Dillenseger
 */
public class SocketSessionObject implements SampleAction, ControlAction, TestAction, SessionObjectAction, DataProvider
{
	static private final String LOCAL_ADDRESS_ARG = "localaddress";
	static private final String TARGET_HOST_ARG = "targethost";
	static private final String TARGET_PORT_ARG = "targetport";
	static private final String BUFFER_NAME_ARG = "buffername";
	static private final String DATA_STRING_ARG = "datastring";
	static private final String CHRONO_RESULT = "result";
	static private final String CHRONO_COMMENT = "comment";
	static private final String CHRONO_SUCCESS = "successful";
	static private final int IS_OPEN_TEST = 0;
	static private final int IS_NOT_OPEN_TEST = 1;
	static private final int CHRONO_ON_CONTROL = 2;
	static private final int CHRONO_OFF_CONTROL = 3;
	static private final int READ_BYTESTRING_SAMPLE = 0;
	static private final int READ_UTFSTRING_SAMPLE = 1;
	static private final int WRITE_BYTESTRING_SAMPLE = 2;
	static private final int WRITE_UTFSTRING_SAMPLE = 3;
	static private final int CHRONO_SAMPLE = 4;
	static private final int OPEN_SAMPLE = 5;
	static private final int CLOSE_SAMPLE = 6;

	static private final int BUFFER_SIZE = 4096;

	private Socket sock = null;
	private InetAddress localhost;
	private InetSocketAddress target;
	private Map buffers = new HashMap();
	private byte[] buffer = new byte[BUFFER_SIZE];
	private boolean chronoIsOn = false;
	private int chronoDuration = 0;
	private long chronoStartDate;


	/**
	 * Build a new SessionObject for this plugin
	 * @param params The table containing all the parameters
	 */
	public SocketSessionObject(Hashtable params)
		throws ClifException
	{
		// local address setting
		String value = (String)params.get(LOCAL_ADDRESS_ARG);
		if (value != null && value.length() > 0)
		{
			try
			{
				localhost = InetAddress.getByAddress(value, new byte[4]);
			}
			catch (Exception ex)
			{
				throw new ClifException("ISAC can't create Socket session object because the specified local IP address is not valid: " + value, ex);
			}
		}
		else
		{
			localhost = Network.getInetAddress(null);
		}
		if (localhost == null)
		{
			throw new ClifException("ISAC can't create Socket session object because no local network interface is configured");
		}
		// target address setting
		String targetHost = (String)params.get(TARGET_HOST_ARG);
		if (targetHost == null || targetHost.length() == 0)
		{
			throw new ClifException("ISAC can't create Socket session object because no target host is specified");
		}
		String targetPort = (String)params.get(TARGET_PORT_ARG);
		try
		{
			target = new InetSocketAddress(targetHost, Integer.parseInt(targetPort));
		}
		catch (Exception ex)
		{
			throw new ClifException("ISAC can't create Socket session object because target port is unspecified or invalid", ex);
		}
	}


	/**
	 * This constructor is used to clone the specified session object
	 *
	 * @param toClone
	 *            The session object to clone
	 */
	private SocketSessionObject(SocketSessionObject toClone)
	{
		this.localhost = toClone.localhost;
		this.target = toClone.target;
	}


	private ActionEvent openSocket(ActionEvent report)
	{
		report.setDate(System.currentTimeMillis());
		if (chronoIsOn && chronoStartDate == -1)
		{
			chronoStartDate = report.getDate();
		}
		try
		{
			sock = new Socket(target.getAddress(), target.getPort(), localhost, 0);
			report.duration = (int)(System.currentTimeMillis() - report.getDate());
			chronoDuration += report.duration;
			report.successful = true;
			report.result = localhost + " connected to " + target;
			return chronoIsOn ? null : report;
		}
		catch (Exception ex)
		{
			report.successful = false;
			report.comment = "ISAC SocketInjector can't connect " + localhost + " to " + target;
			report.result = ex.toString();
			return report;
		}
	}


	private void closeSocket()
	{
		if (sock != null)
		{
			try
			{
				sock.close();
				sock = null;
			}
			catch (Exception ex)
			{
				throw new RuntimeException("ISAC SocketInjector could not close socket", ex);
			}
		}
	}


	private ActionEvent closeSocket(ActionEvent report)
	{
		if (sock != null)
		{
			report.setDate(System.currentTimeMillis());
			if (chronoIsOn && chronoStartDate == -1)
			{
				chronoStartDate = report.getDate();
			}
			try
			{
				sock.close();
				report.duration = (int)(System.currentTimeMillis() - report.getDate());
				chronoDuration += report.duration;
				sock = null;
				report.successful = true;
				report.result = localhost + " disconnected from " + target;
				return chronoIsOn ? null : report;
			}
			catch (Exception ex)
			{
				report.successful = false;
				report.comment = "ISAC SocketInjector can't disconnect " + localhost + " from " + target;
				report.result = ex.toString();
				return report;
			} 
		}
		else
		{
			report.successful = false;
			report.comment = "ISAC SocketInjector can't close unopen socket connection";
			report.result = "ignored";
			return null;
		}
	}


	private ActionEvent readSocket(ActionEvent report, boolean asBytes, String bufferName)
	{
		try
		{
			int size = 0;
			StringBuffer data = new StringBuffer();
			report.setDate(System.currentTimeMillis());
			if (chronoIsOn && chronoStartDate == -1)
			{
				chronoStartDate = report.getDate();
			}
			if (asBytes)
			{
				int n;
				InputStream in = sock.getInputStream();
				while ((n = in.read(buffer)) != -1)
				{
					size += n;
					if (bufferName != null)
					{
						data.append(new String(buffer, 0, n));
					}
				}
			}
			else
			{
				DataInputStream in = new DataInputStream(sock.getInputStream());
				String str;
				try
				{
					while ((str = in.readUTF()) != null)
					{
						size += str.length();
						if (bufferName != null)
						{
							data.append(str);
						}
					}
				}
				catch (EOFException ex)
				{
				}
			}
			report.duration = (int)(System.currentTimeMillis() - report.getDate());
			chronoDuration += report.duration;
			report.successful = true;
			report.comment = size + " bytes read";
			report.result = "OK";
			if (bufferName != null)
			{
				buffers.put(bufferName, data.toString());
			}
			return chronoIsOn ? null : report;
		}
		catch (Exception ex)
		{
			report.successful = false;
			report.comment = ex.getMessage();
			report.result = ex.toString();
			return report;
		}
	}


	private ActionEvent writeSocket(ActionEvent report, boolean asBytes, String data)
	{
		try
		{
			report.setDate(System.currentTimeMillis());
			if (chronoIsOn && chronoStartDate == -1)
			{
				chronoStartDate = report.getDate();
			}
			DataOutputStream out = new DataOutputStream(sock.getOutputStream());
			if (asBytes)
			{
				out.writeBytes(data);
			}
			else
			{
				out.writeUTF(data);
			}
			out.write(new byte[] { '\r', '\n' }); 
			out.flush();
			report.duration = (int)(System.currentTimeMillis() - report.getDate());
			chronoDuration += report.duration;
			report.successful = true;
			report.comment = data.length() + " chars written";
			report.result = "OK";
			return chronoIsOn ? null : report;
		}
		catch (Exception ex)
		{
			report.successful = false;
			report.comment = ex.getMessage();
			report.result = ex.toString();
			return report;
		}
	}


	///////////////////////////
	// TestAction interface //
	///////////////////////////


	public boolean doTest(int number, Map params)
		throws IsacRuntimeException
	{
		switch (number)
		{
			case IS_OPEN_TEST:
				return sock != null;
			case IS_NOT_OPEN_TEST:
				return sock == null;
			default:
				throw new Error("Fatal error in ISAC's SocketInjector plug-in: unknown test identifier " + number);
		}
	}


	////////////////////////////
	// ControlAction interface //
	////////////////////////////


	public void doControl(int number, Map params)
	{
		switch (number)
		{
			case CHRONO_ON_CONTROL:
				chronoIsOn = true;
				chronoStartDate = -1;
				chronoDuration = 0;
				break;
			case CHRONO_OFF_CONTROL:
				chronoIsOn = false;
				break;
			default:
				throw new Error("Fatal error in ISAC's SocketInjector plug-in: unknown control identifier " + number);
		}
	}


	////////////////////////////
	// SampleAction interface //
	////////////////////////////


	public ActionEvent doSample(int number, Map params, ActionEvent report)
	{
		switch (number)
		{
			case READ_BYTESTRING_SAMPLE:
			{
				report.type = "Read byte string";
				String bufferName = (String)params.get(BUFFER_NAME_ARG);
				if (bufferName != null && buffer.length == 0)
				{
					bufferName = null;
				}
				return readSocket(report, true, bufferName);
			}
			case READ_UTFSTRING_SAMPLE:
			{
				report.type = "Read UTF string";
				String bufferName = (String)params.get(BUFFER_NAME_ARG);
				if (bufferName != null && buffer.length == 0)
				{
					bufferName = null;
				}
				return readSocket(report, false, bufferName);
			}
			case WRITE_BYTESTRING_SAMPLE:
				report.type = "Write byte string";
				return writeSocket(report, true, (String)params.get(DATA_STRING_ARG));
			case WRITE_UTFSTRING_SAMPLE:
				report.type = "Write UTF string";
				return writeSocket(report, false, (String)params.get(DATA_STRING_ARG));
			case CHRONO_SAMPLE:
				report.type = "custom chrono section";
				report.comment = (String)params.get(CHRONO_COMMENT);
				report.duration = chronoDuration;
				report.setDate(chronoStartDate);
				report.result = (String)params.get(CHRONO_RESULT);
				String success = (String)params.get(CHRONO_SUCCESS);
				report.successful = success != null && success.length() != 0; 
				return report;
			case OPEN_SAMPLE:
				return openSocket(report);
			case CLOSE_SAMPLE:
				return closeSocket(report);
			default:
				throw new Error("Fatal error in ISAC's SocketInjector plug-in: unknown sample identifier " + number);
		}
	}
	
	////////////////////////////
	// DataProvider interface //
	////////////////////////////


	public String doGet(String bufferName)
	{
		return (String)buffers.get(bufferName);
	}


	///////////////////////////////////
	// SessionObjectAction interface //
	///////////////////////////////////


	public Object createNewSessionObject()
	{
		return new SocketSessionObject(this);
	}


	public void close()
	{
		closeSocket();
	}


	public void reset()
	{
		closeSocket();
		buffers.clear();
		chronoIsOn = false;
	}
}
